from .base_object import BaseObject


class CollaborationAllowlistExemptTarget(BaseObject):
    """Represents a user who is exempted from the collaboration allowlist."""

    _item_type = 'collaboration_whitelist_exempt_target'
