<html>
<head>
    <title>BAM! It's HTML instead of JSON!</title>
</head>
<body>
<div>
    <div>You're probably looking for the information below.</div>
    <div>{{response}}</div>
</div>
</body>
</html>